/*====================================+=====================================+
! File CApp.h                         ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/cmdline.h>
#include <wx/app.h>
#include <wx/filename.h>
#include <wx/snglinst.h>
#include <wx/ipc.h>
#include "matelefree.h"
#include "common/sc_config.h"
#include "CChannel.h"
#include "CPlaylist.h"
/*-------------------------------------------------------------------------*/
class CFrame ;
class wxImageList ;
class CSIServer ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CApp : public wxApp
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      ~CApp() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      bool       m_boo_first_start_ever = false ; // Program never started ?
      wxFileName m_fn_exe ; // Current executable
      wxLocale   m_locale ; // Default main locale object
      /*--( Optimize searches by name and num )----------------------------*/
      t_map_channel_num m_map_channel_num ;
      /*-------------------------------------------------------------------*/
      bool       m_boo_forcefreebox = false ;
      /*--( Force smallest mini display ? )--------------------------------*/
      bool       m_boo_force_minimalest = false ;
      wxString   m_s_program_to_start_on_exit ;
      bool       m_boo_use_vlc_init_str = true ;
      bool       m_boo_loading_channels_from_url = false ;
      /*-------------------------------------------------------------------*/
      str_pk_info m_a_pk_info[ PK_NB ]
      = { { "fbtv", wxTRANSLATE( "TV" ) },                   // PK_FBTV
          { "fbtvb", wxTRANSLATE( "Bouquet basique" ) },     // PK_FBTVB
          { "fbtvo", wxTRANSLATE( "En Option" ) },           // PK_FBTVO
          { "fbtvo_ok", wxTRANSLATE( "En Option accessible" ) },//PK_FBTVO_OK
          { "fbradio", wxTRANSLATE( "Radio" ) },             // PK_FBRADIO
          { "search", wxTRANSLATE( "Search" ) },             // PK_SEARCH
          { "live", wxTRANSLATE( "Live" )},                  // PK_LIVE_NODE
          { CChannelDailymotion::st_get_pl_source(),         // PK_LIVE_DM
            wxTRANSLATE( "Dailymotion" )
          },
          { CChannelTwitch::st_get_pl_source(),              // PK_LIVE_TW
            wxTRANSLATE( "Twitch" )
          },
          { CChannelYoutube::st_get_pl_source(),             // PK_LIVE_YT
            wxTRANSLATE( "Youtube" )
          },
          { CChannelUstream::st_get_pl_source(),             // PK_LIVE_US
            wxTRANSLATE( "Ustream" )
          },
          { CChannelPeriscope::st_get_pl_source(),           // PK_LIVE_PE
            wxTRANSLATE( "Periscope" )
          },
          { CChannelMixer::st_get_pl_source(),               // PK_LIVE_MX
            wxTRANSLATE( "Mixer" )
          },
          { "user", wxTRANSLATE( "User" ) }                  // PK_USER_NODE
        } ;
      /*-------------------------------------------------------------------*/
      wxSingleInstanceChecker *m_sins_checker = nullptr ;
      sc::CSIServer           *m_sins_server  = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxFileName m_fn_vlc ; // vlc executable
      /*-------------------------------------------------------------------*/
      sc::t_map_strstr m_map_image_fn ;
      /*-------------------------------------------------------------------*/
      wxImageList   *m_il_logo = nullptr ;
      t_map_channel m_map_channel ;
      /*-------------------------------------------------------------------*/
      CFrame   *m_frame = nullptr ;
      wxString m_s_media_path ;
      wxString m_s_media_fb_path ;
      wxString m_s_tree_fb_path ;
      wxString m_s_tree_fbtv_path ;
      wxString m_s_tree_live_path ;
      wxString m_s_tree_pl_user_path ;
      /*-------------------------------------------------------------------*/
      const sc::t_lst_string m_co_ls_twitch_header
          = { { "Client-ID: "
                // Matelefree client_id : "asvncavqk0no3newx9ukaz4i230vqz"
                // Twitch web player
                "kimne78kx3ncx6brgo4mv6wki5h1ko"
              }
            } ;

   /*----------------------------------------------------------------------*/
   public :
      /*--------------------------------------------------------------------+
      ! Serialized member variables : the "m" is upper-case                 !
      +--------------------------------------------------------------------*/
      CConfigRect   M_rec_main ;
      CConfigRect   M_rec_main_mini ;
      CConfigBool   M_boo_mini_display ;
      CConfigBool   M_boo_mini_display_caption ;
      CConfigBool   M_boo_stay_on_top ;
      CConfigString M_s_bopl_sel_name ;
      CConfigBool   M_boo_freebox ;
      CConfigBool   M_boo_mute ;
      CConfigInt    M_i_max_nb_media ;
      CConfigBool   M_boo_new_media_active ;
      CConfigInt    M_i_check_media_delay ;
      CConfigInt    M_i_volume ;
      /*-------------------------------------------------------------------*/
      CConfigRect   M_rec_prefs ;
      CConfigString M_s_lang ; // Canonical name: en_US ...
      CConfigString M_s_vlc_path ;
      CConfigString M_s_snapshot_path ;
      CConfigString M_s_vlc_init_dm ;
      CConfigString M_s_vlc_init_tw ;
      CConfigString M_s_vlc_init_yt ;
      CConfigString M_s_vlc_init_us ;
      CConfigString M_s_vlc_init_pe ;
      CConfigString M_s_vlc_init_mx ;
      CConfigString M_s_vlc_init_other ;
      CConfigString M_s_vlc_init_all ;
      CConfigString M_s_deinterlace ;
      CConfigInt    M_i_chkupd_freq ;
      CConfigString M_s_chkupd_last ; // YYYY-MM-DD (ISO)
      CConfigString M_s_def_quality ;
      CConfigInt    M_i_viewers_delay ;
      CConfigInt    M_i_retry_delay ; // Time between two get stream retries
      CConfigBool   M_boo_mute_deact ;
      CConfigBool   M_boo_startup_play ;
      CConfigBool   M_boo_single_instance ;
      CConfigString M_s_proxy_server ;
      CConfigString M_s_startup_play_list ; // Channels at program stop
      CConfigString M_s_startup_play_active ; // Active channel at stop
      CConfigString M_s_play_hist ;
      /*-------------------------------------------------------------------*/
      CConfigRect   M_rec_progress ;
      CConfigRect   M_rec_lastversion ;
      CConfigRect   M_rec_newversion ;
      CConfigRect   M_rec_plname ;
      CConfigRect   M_rec_chname ;
      CConfigRect   M_rec_addlive ;
      /*-------------------------------------------------------------------*/
      CConfigString M_s_main_perspective ;
      CConfigString M_s_bopl_pl_user_list ;
      CConfigString M_s_bopl_expanded ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxFont  m_font_pl_normal ;
      wxFont  m_font_pl_played ;
      wxFont  m_font_pl_played_active ;
      wxFont  m_font_pl_hil_onair ;
      wxColor m_color_pl_onair ;
      wxColor m_color_pl_hil_onair ;
      wxColor m_color_pl_alt ;
      wxColor m_color_pl_alt_onair ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool OnInit() override ;
      virtual void OnInitCmdLine( wxCmdLineParser &parser ) override ;
      virtual bool OnCmdLineParsed( wxCmdLineParser &parser ) override ;
      /*-------------------------------------------------------------------*/
      int actions_for_install_process() ;
      wxString get_exe_path() const { return m_fn_exe.GetPath() ; }
      wxString get_exe_fullpath() const { return m_fn_exe.GetFullPath() ; }
      wxString get_exe_fullname() const { return m_fn_exe.GetFullName() ; }
      bool get_force_minimalest() const { return m_boo_force_minimalest ; }
      bool get_use_vlc_init_str() const { return m_boo_use_vlc_init_str ; }
      CChannel *get_channel( const wxString &s_channel )
      {   auto it = m_map_channel.find( s_channel ) ;
          return it == m_map_channel.end() ? nullptr : it->second ;
      }
      bool get_fb_access() const
      {  return m_boo_forcefreebox || M_boo_freebox.get() ; }
      const wxString &get_program_to_start_on_exit() const
      {  return m_s_program_to_start_on_exit ; }
      void set_program_to_start_on_exit( const wxString &s_program )
      {  m_s_program_to_start_on_exit = s_program ; }
      void restart_on_exit()
      {  m_s_program_to_start_on_exit = get_exe_fullpath() ; }
      void no_start_on_exit()
      {  m_s_program_to_start_on_exit.clear() ; }
      /*-------------------------------------------------------------------*/
      void init_fn_vlc() ;
      void init_imagelist() ;
      void init_translations( const wxString &s_canonical_name
                                           = wxEmptyString
                            ) ;
      CChannel *find_channel( const wxString &s_channel ) const ;
      CChannel *find_channel( int i_num ) const ;
      void init_config_info_to_keep() ;
      void open_config_file( const wxString &s_file, const int i_version ) ;
      void init_config_info() ;
      void save_config_info() ;
      /*-------------------------------------------------------------------*/
      int add_channel( CChannel *p_channel ) ;
      void add_channel_m3u( const wxString &s_pl, const wxString &s_name,
                            const wxString &s_url_m3u,
                            sc::t_vec_str &vs_channel
                          ) ;
      void add_channel( CChannelDynamicKnown::e_cdtk cdtk,
                        const wxString &s_pl,
                        const wxString &s_id, const wxString &s_name,
                        sc::t_vec_str &vs_channel
                      ) ;
      void add_channel_dailymotion( const wxString &s_pl,
                                    const wxString &s_id,
                                    const wxString &s_name,
                                    sc::t_vec_str &vs_channel
                                  ) ;
      void add_channel_youtube( const wxString &s_pl,
                                const wxString &s_id,
                                const wxString &s_name,
                                sc::t_vec_str &vs_channel
                              ) ;
      int del_channel( const sc::t_vec_str &vs_channel ) ;
      int rename_channel( const wxString &s_from_full,
                          const wxString &s_to_short
                        ) ;
      int load_channels( const wxString &s_origin, const wxString &s_pl_name,
                         wxInputStream *p_is,
                         sc::t_vec_str &vs_channel,
                         wxString *p_s_label = nullptr
                       ) ;
      int load_channels_from_file( const wxString &s_filename,
                                   const wxString &s_pl_name,
                                   sc::t_vec_str &vs_channel,
                                   wxString *p_s_label = nullptr
                                 ) ;
      int load_channels_from_file( const wxString &s_filename,
                                   sc::t_vec_str &vs_channel
                                 )
      {  return load_channels_from_file( s_filename, wxEmptyString,
                                         vs_channel
                                       ) ;
      }
      int load_channels_from_url( const wxString &s_url,
                                  const wxString &s_pl_name,
                                  sc::t_vec_str &vs_channel,
                                  wxString *p_s_label = nullptr
                                ) ;
      int load_channels_freebox_tv( sc::t_vec_str &vs_channel ) ;
      void init_image_fn() ;
      void init_channels() ;
      void init_playlist_frtv( sc::t_vec_str &vs_channel ) ;
      void init_playlists() ;
      void load_channel_config_info() ;
      void save_channel_config_info() ;
      void save_playlist_config_info() ;

      /*-------------------------------------------------------------------*/
      const wxString &get_pk_name( int sd )
      {  return m_a_pk_info[ sd ].m_s_name ; }
      const wxString &get_pk_label( int sd )
      {  return m_a_pk_info[ sd ].m_s_label ; }
      CPlaylist *get_pk_pl( int sd )
      {  return m_a_pk_info[ sd ].m_pl ; }
      void set_pk_pl( int sd, CPlaylist *pl )
      {  m_a_pk_info[ sd ].m_pl = pl ; }
      e_pl_known find_pl_known( const wxString &s_name ) const ;
      bool is_pk_name_reserved( const wxString &s_name ) const ;
      int single_instance() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
DECLARE_APP( CApp )
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                            End of file CApp.h                             !
+==========================================================================*/
