/*====================================+=====================================+
! File CBookPlaylist.h                ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/treectrl.h>
#include "matelefree.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CTreeItemDataPl : public wxTreeItemData
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      explicit CTreeItemDataPl( CPlaylist *pl ) : m_pl( pl )
      {  ; }

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CPlaylist *m_pl = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPlaylist *get_pl() const { return m_pl ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CBookPlaylist : public wxTreeCtrl
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      using t_fnc_pl = void (*)( CPlaylist *pl ) ;
      /*-------------------------------------------------------------------*/
      CBookPlaylist( wxWindow *parent, const wxString &s_name ) ;
      ~CBookPlaylist() ;

   /*----------------------------------------------------------------------*/
   private :
      /*--( Associated window where the playlist is displayed )------------*/
      wxPanel      *m_panel_pl = nullptr ;
      wxBoxSizer   *m_sizer_pl = nullptr ;
      /*--( Currently dragged tree item )----------------------------------*/
      wxTreeItemId m_ti_drag ;
      /*--( Playlists image list )-----------------------------------------*/
      wxImageList  *m_il_pl = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void OnTreeSelChanged( wxTreeEvent &event ) ;
      void OnTreeBeginDrag( wxTreeEvent &event ) ;
      void OnTreeEndDrag( wxTreeEvent &event ) ;
      void OnTreeKeyDown( wxTreeEvent &event ) ;
      /*-------------------------------------------------------------------*/
      wxImageList *get_il_pl() { return m_il_pl ; }
      CTreeItemDataPl *get_tidpl( const wxTreeItemId &ti )
      {  return ti.IsOk()
                ? static_cast< CTreeItemDataPl * >( GetItemData( ti ) )
                : nullptr ;
      }
      wxWindow *get_window_pl() { return m_panel_pl ; }
      CPlaylist *get_playlist( const wxTreeItemId &ti )
      {  CTreeItemDataPl *p_tidpl = get_tidpl( ti ) ;
         return p_tidpl == nullptr ? nullptr : p_tidpl->get_pl() ;
      }
      CPlaylist *get_playlist_active()
      {  return get_playlist( GetSelection() ) ; }
      bool is_parent_user_node( const wxTreeItemId &ti )
      {  return    get_playlist( GetItemParent( ti ) )
                == wxGetApp().get_pk_pl( PK_USER_NODE ) ;
      }
      wxTreeItemId find_ti_name( const wxString &s_pl_name,
                                 wxTreeItemId ti_start = wxTreeItemId()
                               ) ;
      wxTreeItemId get_first_ti() const
      {  wxTreeItemIdValue tiv_cookie ;
         return GetFirstChild( GetRootItem(), tiv_cookie ) ;
      }
      wxTreeItemId get_last_ti() const
      {  return GetLastChild( GetRootItem() ) ; }
      CPlaylist *get_first_pl()
      {  return get_playlist( get_first_ti() ) ; }
      /*-------------------------------------------------------------------*/
      int get_pl_children( e_pl_known pl_known, t_vec_pl &vec_pl ) ;
      wxString get_pl_user_list() ;
      int del_channel_pl( const sc::t_vec_str &vs_channel ) ;
      wxString get_expanded( wxTreeItemId ti_start = wxTreeItemId() ) ;
      bool swap_ti_values( const wxTreeItemId &ti1,
                           const wxTreeItemId &ti2
                         ) ;
      void move_up( const wxTreeItemId &ti ) ;
      void move_down( const wxTreeItemId &ti ) ;
      /*-------------------------------------------------------------------*/
      wxTreeItemId get_selection() const
      {  return GetSelection() ; }
      wxString get_selection_name()
      {  const wxTreeItemId &ti = get_selection() ;
         if( ti.IsOk() ) { return get_playlist( ti )->get_name() ; }
         return wxEmptyString ;
      }
      void set_selection( const wxTreeItemId &ti, bool boo_sel = true )
      {  SelectItem( ti, boo_sel ) ; }
      void set_selection( const wxString &s_name, bool boo_sel = true )
      {  wxTreeItemId ti = find_ti_name( s_name ) ;
         if( ti.IsOk() ) { set_selection( ti, boo_sel ) ; }
      }
      void make_selection_visible()
      {  if( GetSelection().IsOk() ) { EnsureVisible( GetSelection() ) ; } }
      void expand_list( const wxString &s_names ) ;
      wxTreeItemId get_hittest( const wxPoint &pt )
      {  return HitTest( pt ) ; }
      wxTreeItemId get_hittest( const wxPoint &pt, int &i_flags )
      {  return HitTest( pt, i_flags ) ; }
      CPlaylist *get_pl_hittest( const wxPoint &pt )
      {  return get_playlist( HitTest( pt ) ) ; }
      wxTreeItemId add_pl( const wxTreeItemId &ti_parent, size_t sz_pos,
                           CPlaylist *pl
                         ) ;
      wxTreeItemId add_pl( CPlaylist *pl, size_t sz_pos = -1 ) ;
      void rename_pl( const wxTreeItemId &ti ) ;
      void change_window_pl( wxWindow *win_new ) ;
      void remove_pl( const wxTreeItemId &ti ) ;
      void delete_pl( const wxTreeItemId &ti ) ;
      void context_menu( wxTreeEvent &event ) ;
      wxTreeItemId find_child( const wxTreeItemId &ti_parent,
                               const wxString &s_text
                             ) ;
      wxTreeItemId make_path( const wxString &s_path ) ;
   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      void exec_on_tree_pl( t_fnc_pl fnc_pl, wxTreeItemId ti_start ) ;
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void exec_on_tree_pl( t_fnc_pl fnc_pl )
      {  exec_on_tree_pl( fnc_pl, GetRootItem() ) ; }
      void init_list_all()
      {  exec_on_tree_pl( []( CPlaylist *pl ) { pl->init_list() ; },
                          GetRootItem()
                        ) ;
      }
      void init_list_parents( const wxTreeItemId &ti ) ;
      void init_list_parents( const CPlaylist *pl )
      {  init_list_parents( pl->get_ti() ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file CBookPlaylist.h                        !
+==========================================================================*/
