/*====================================+=====================================+
! File CDlgChName.cpp                 ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include "CFrame.h"
#include "CDlgChName.h"
#include "CPlaylist.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CDlgChName::CDlgChName( wxWindow *parent,
                        const wxString &s_title,
                        const CChannel *p_channel
                      )
          : wxDialog( parent, wxID_ANY, s_title,
                      wxDefaultPosition, wxDefaultSize,
                      wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                    ),
            m_p_channel( p_channel )
{
   /*--( Build UI )--------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   wxFlexGridSizer *fgsz = new wxFlexGridSizer( 2, 0, 0 ) ;
   fgsz->AddGrowableCol( 1 ) ;
   fgsz->Add( new wxStaticText( this, wxID_ANY, _( "Name" ) ),
              0, wxALIGN_CENTER_VERTICAL | wxALL, 5
            ) ;

   m_txt_name = new wxTextCtrl( this, wxID_ANY, wxEmptyString,
                                wxDefaultPosition, wxDefaultSize,
                                wxTE_PROCESS_ENTER
                              ) ;
   m_txt_name->SetMinSize( wxSize( 300, -1 ) ) ;
   fgsz->Add( m_txt_name, 1, wxALL | wxEXPAND, 5 ) ;

   fgsz->AddStretchSpacer( 1 ) ;
   m_stx_errmsg = new wxStaticText( this, wxID_ANY, wxEmptyString ) ;
   fgsz->Add( m_stx_errmsg, 0, wxALL, 5 ) ;

   bsz_gen->Add( fgsz, 0, wxALL | wxEXPAND, 5 ) ;

   /*--( Button bar )------------------------------------------------------*/
   bsz_gen->Add( CreateButtonSizer( wxOK | wxCANCEL ),
                 0, wxALL | wxEXPAND, 5
               ) ;
   auto bt = FindWindow( wxID_OK ) ;
   if( bt != nullptr )
   {  bt->Bind( wxEVT_COMMAND_BUTTON_CLICKED, &CDlgChName::OnBtOk, this ) ; }

   SetSizer( bsz_gen ) ;

   /*----------------------------------------------------------------------*/
   m_txt_name->Bind( wxEVT_COMMAND_TEXT_ENTER,
                     &CDlgChName::OnTextEnter, this
                   ) ;
   m_txt_name->SetValue( m_p_channel->get_short_name() ) ;
   m_txt_name->SetFocus() ;
   m_stx_errmsg->SetForegroundColour( *wxRED ) ;
   /*----------------------------------------------------------------------*/
   sc::init_tlw_rect( this, wxGetApp().M_rec_chname ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgChName::~CDlgChName()
{
   /*----------------------------------------------------------------------*/
   sc::save_tlw_rect( this, wxGetApp().M_rec_chname ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgChName::OnTextEnter( wxCommandEvent &event )
{
   /*--( Exit only if the name is ok )-------------------------------------*/
   if( check_name() ) { EndModal( wxID_OK ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgChName::OnBtOk( wxCommandEvent &event )
{
   /*--( Exit only if the name is ok )-------------------------------------*/
   if( check_name() ) { EndModal( wxID_OK ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CDlgChName::check_name()
{
   /*----------------------------------------------------------------------*/
   m_s_new_name = m_txt_name->GetValue().Trim().Trim( false ) ;
   /*----------------------------------------------------------------------*/
   if( m_s_new_name.empty() )
   {  m_stx_errmsg->SetLabelText( _( "A channel must have a name" ) ) ;
      return false ;
   }
   /*--( Same name is ok )-------------------------------------------------*/
   if( m_s_new_name == m_p_channel->get_short_name() )
   {  return true ; }
   /*--( The string must be ok to be part of an "ini" file )---------------*/
   wxString s_err_msg ;
   if( mtf::check_ini_entry_name( m_s_new_name, s_err_msg ) != 0 )
   {  m_stx_errmsg->SetLabelText( s_err_msg ) ;
      return false ;
   }
   /*--( Name must be unique )---------------------------------------------*/
   CChannel *p_channel
       = wxGetApp().find_channel( m_p_channel->build_name( m_s_new_name ) ) ;
   if( p_channel != nullptr )
   {  m_stx_errmsg->SetLabelText(
                        wxString::Format( _( "Name \"%s\" is already used" ),
                                          m_s_new_name
                                        )
                                ) ;
      return false ;
   }
   /*----------------------------------------------------------------------*/
   return true ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgChName::get_modal( wxString &s_new_name )
{
   /*--( Same name will be seen as if the edition had been "canceled" )----*/
   if(    ShowModal() == wxID_OK
       && m_s_new_name != m_p_channel->get_short_name()
     )
   {  s_new_name = m_s_new_name ; }
   /*----------------------------------------------------------------------*/
   return GetReturnCode() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CDlgChName.cpp                          !
+==========================================================================*/
