/*====================================+=====================================+
! File matelefree.cpp                 ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/clipbrd.h>
#include <wx/filename.h>
#include <wx/txtstrm.h>
#include <curl/curl.h>
#include "matelefree.h"
#include "common/sc_download.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
wxString mtf::unspace( wxString s )
{
   /*----------------------------------------------------------------------*/
   int i_pos ;
   while( ( i_pos = s.find( ' ' ) ) >= 0 ) { s.erase( i_pos, 1 ) ; }
   /*----------------------------------------------------------------------*/
   return s ;
   /*----------------------------------------------------------------------*/
}

/*--( "Computation" of the non accented version of a character )-----------*/
static wxUniChar st_unaccent( const wxUniChar &c )
{
   /*--( Ascii means non accented ... )------------------------------------*/
   if( c.IsAscii() ) { return c ; }
   /*----------------------------------------------------------------------*/
   switch( c.GetValue() )
   {
      /*-------------------------------------------------------------------*/
      case 0x01E2 : case 0x01FC :
         return 0x00C6 ;
      case 0x00C0 : case 0x00C1 : case 0x00C2 : case 0x00C3 : case 0x00C4 :
      case 0x00C5 : case 0x0100 : case 0x0102 : case 0x0104 : case 0x01CD :
      case 0x01DE : case 0x01E0 : case 0x01FA : case 0x0200 : case 0x0202 :
      case 0x0226 :
         return 'A' ;
      case 0x00C7 : case 0x0106 : case 0x0108 : case 0x010A : case 0x010C :
      case 0x0187 :
         return 'C' ;
      case 0x010E : case 0x0110 :
         return 'D' ;
      case 0x00C8 : case 0x00C9 : case 0x00CA : case 0x00CB : case 0x0112 :
      case 0x0114 : case 0x0116 : case 0x0118 : case 0x011A : case 0x0204 :
      case 0x0206 : case 0x0228 :
         return 'E' ;
      case 0x011C : case 0x011E : case 0x0120 : case 0x0122 : case 0x01E4 :
      case 0x01E6 : case 0x01F4 :
         return 'G' ;
      case 0x0124 : case 0x0126 : case 0x021E :
         return 'H' ;
      case 0x00CC : case 0x00CD : case 0x00CE : case 0x00CF : case 0x0128 :
      case 0x012A : case 0x012C : case 0x012E : case 0x0130 : case 0x01CF :
      case 0x0208 : case 0x020A :
         return 'I' ;
      case 0x0134 :
         return 'J' ;
      case 0x0136 : case 0x01E8 :
         return 'K' ;
      case 0x0139 : case 0x013B : case 0x013D : case 0x013F : case 0x0141 :
         return 'L' ;
      case 0x00D1 : case 0x0143 : case 0x0145 : case 0x0147 : case 0x014A :
      case 0x01F8 :
         return 'N' ;
      case 0x00D2 : case 0x00D3 : case 0x00D4 : case 0x00D5 : case 0x00D6 :
      case 0x014C : case 0x014E : case 0x0150 : case 0x01A0 : case 0x01D1 :
      case 0x01FE : case 0x020C : case 0x020E : case 0x022A : case 0x022C :
      case 0x022E : case 0x0230 :
         return 'O' ;
      case 0x01EA : case 0x01EC :
         return 'Q' ;
      case 0x0154 : case 0x0156 : case 0x0158 :
         return 'R' ;
      case 0x008A : case 0x015A : case 0x015C : case 0x015E : case 0x0160 :
         return 'S' ;
      case 0x0162 : case 0x0164 : case 0x0166 : case 0x021A :
         return 'T' ;
      case 0x00D9 : case 0x00DA : case 0x00DB : case 0x00DC : case 0x0168 :
      case 0x016A : case 0x016C : case 0x016E : case 0x0170 : case 0x0172 :
      case 0x01AF : case 0x01D3 : case 0x01D5 : case 0x01D7 : case 0x01D9 :
      case 0x01DB : case 0x0214 : case 0x0216 :
         return 'U' ;
      case 0x00DD : case 0x009F : case 0x0176 : case 0x0178 : case 0x0232 :
         return 'Y' ;
      case 0x0174 :
         return 'W' ;
      case 0x008E : case 0x0179 : case 0x017B : case 0x017D : case 0x0224 :
         return 'Z' ;
      case 0x01E3 : case 0x01FD :
         return 0x00E6 ;
      case 0x00E0 : case 0x00E1 : case 0x00E2 : case 0x00E3 : case 0x00E4 :
      case 0x00E5 : case 0x0101 : case 0x0103 : case 0x0105 : case 0x01CE :
      case 0x01DF : case 0x01E1 : case 0x01FB : case 0x0201 : case 0x0203 :
      case 0x0227 :
         return 'a' ;
      case 0x00E7 : case 0x0107 : case 0x0109 : case 0x010B : case 0x010D :
      case 0x0188 :
         return 'c' ;
      case 0x010F : case 0x0111 :
         return 'd' ;
      case 0x00E8 : case 0x00E9 : case 0x00EA : case 0x00EB : case 0x0113 :
      case 0x0115 : case 0x0117 : case 0x0119 : case 0x011B : case 0x0205 :
      case 0x0207 : case 0x0229 :
         return 'e' ;
      case 0x011D : case 0x011F : case 0x0121 : case 0x0123 : case 0x01E5 :
      case 0x01E7 : case 0x01F5 :
         return 'g' ;
      case 0x0125 : case 0x0127 : case 0x021F :
         return 'h' ;
      case 0x00EC : case 0x00ED : case 0x00EE : case 0x00EF : case 0x0129 :
      case 0x012B : case 0x012D : case 0x012F : case 0x01D0 : case 0x0209 :
      case 0x020B :
         return 'i' ;
      case 0x0135 : case 0x01F0 :
         return 'j' ;
      case 0x0137 : case 0x0138 : case 0x01E9 :
         return 'k' ;
      case 0x013A : case 0x013C : case 0x013E : case 0x0140 : case 0x0142 :
         return 'l' ;
      case 0x00F1 : case 0x0144 : case 0x0146 : case 0x0148 : case 0x0149 :
      case 0x014B : case 0x01F9 :
         return 'n' ;
      case 0x00F2 : case 0x00F3 : case 0x00F4 : case 0x00F5 : case 0x00F6 :
      case 0x014D : case 0x014F : case 0x0151 : case 0x01A1 : case 0x01D2 :
      case 0x01FF : case 0x020D : case 0x020F : case 0x022B : case 0x022D :
      case 0x022F : case 0x0231 :
         return 'o' ;
      case 0x01EB : case 0x01ED :
         return 'q' ;
      case 0x0155 : case 0x0157 : case 0x0159 :
         return 'r' ;
      case 0x009A : case 0x015B : case 0x015D : case 0x015F : case 0x0161 :
         return 's' ;
      case 0x0163 : case 0x0165 : case 0x0167 : case 0x021B :
         return 't' ;
      case 0x00F9 : case 0x00FA : case 0x00FB : case 0x00FC : case 0x0169 :
      case 0x016B : case 0x016D : case 0x016F : case 0x0171 : case 0x0173 :
      case 0x01B0 : case 0x01D4 : case 0x01D6 : case 0x01D8 : case 0x01DA :
      case 0x01DC : case 0x0215 : case 0x0217 :
         return 'u' ;
      case 0x00FD : case 0x00FF : case 0x0177 : case 0x0233 :
         return 'y' ;
      case 0x0175 :
         return 'w' ;
      case 0x009E : case 0x017A : case 0x017C : case 0x017E : case 0x0225 :
         return 'z' ;
      /*-------------------------------------------------------------------*/
   } ;

   /*----------------------------------------------------------------------*/
   return c ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::unaccent( wxString s )
{
   /*----------------------------------------------------------------------*/
   const int co_i_nb = static_cast< int >( s.size() ) ;
   /*----------------------------------------------------------------------*/
   for( int i_num = 0 ; i_num < co_i_nb ; ++i_num )
   {  s[ i_num ] = st_unaccent( s[ i_num ] ) ; }
   /*----------------------------------------------------------------------*/
   return s ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::prepare_string( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   wxString s_prep = mtf::unspace( mtf::unaccent( s ).Lower() ) ;
   /*--( Convert largest list of non authorized file name characters )-----*/
   int i_pos = 0 ;
   while( ( i_pos = s_prep.find_first_of( R"=(\/:*?"<>|)=", i_pos ) ) >= 0 )
   {  s_prep[ i_pos ] = '-' ; }
   /*----------------------------------------------------------------------*/
   return s_prep ;
   /*----------------------------------------------------------------------*/
}

/*--( Basic html chars conversion )----------------------------------------*/
wxString mtf::unhtml( const wxString &s_in )
{
   /*----------------------------------------------------------------------*/
   wxString s_out ;
   s_out.reserve( s_in.size() ) ;
   /*----------------------------------------------------------------------*/
   static sc::t_map_strint st_map_char_seq
   = { { "AElig"   ,  198 }, { "Aacute"  ,  193 }, { "Acirc"   ,  194 },
       { "Agrave"  ,  192 }, { "Alpha"   ,  913 }, { "Aring"   ,  197 },
       { "Atilde"  ,  195 }, { "Auml"    ,  196 }, { "Beta"    ,  914 },
       { "Ccedil"  ,  199 }, { "Chi"     ,  935 }, { "Dagger"  , 8225 },
       { "Delta"   ,  916 }, { "ETH"     ,  208 }, { "Eacute"  ,  201 },
       { "Ecirc"   ,  202 }, { "Egrave"  ,  200 }, { "Epsilon" ,  917 },
       { "Eta"     ,  919 }, { "Euml"    ,  203 }, { "Gamma"   ,  915 },
       { "Iacute"  ,  205 }, { "Icirc"   ,  206 }, { "Igrave"  ,  204 },
       { "Iota"    ,  921 }, { "Iuml"    ,  207 }, { "Kappa"   ,  922 },
       { "Lambda"  ,  923 }, { "Mu"      ,  924 }, { "Ntilde"  ,  209 },
       { "Nu"      ,  925 }, { "OElig"   ,  338 }, { "Oacute"  ,  211 },
       { "Ocirc"   ,  212 }, { "Ograve"  ,  210 }, { "Omega"   ,  937 },
       { "Omicron" ,  927 }, { "Oslash"  ,  216 }, { "Otilde"  ,  213 },
       { "Ouml"    ,  214 }, { "Phi"     ,  934 }, { "Pi"      ,  928 },
       { "Prime"   , 8243 }, { "Psi"     ,  936 }, { "Rho"     ,  929 },
       { "Scaron"  ,  352 }, { "Sigma"   ,  931 }, { "THORN"   ,  222 },
       { "Tau"     ,  932 }, { "Theta"   ,  920 }, { "Uacute"  ,  218 },
       { "Ucirc"   ,  219 }, { "Ugrave"  ,  217 }, { "Upsilon" ,  933 },
       { "Uuml"    ,  220 }, { "Xi"      ,  926 }, { "Yacute"  ,  221 },
       { "Yuml"    ,  376 }, { "Zeta"    ,  918 }, { "aacute"  ,  225 },
       { "acirc"   ,  226 }, { "acute"   ,  180 }, { "aelig"   ,  230 },
       { "agrave"  ,  224 }, { "alefsym" , 8501 }, { "alpha"   ,  945 },
       { "amp"     ,   38 }, { "and"     , 8743 }, { "ang"     , 8736 },
       { "apos"    ,   39 }, { "aring"   ,  229 }, { "asymp"   , 8776 },
       { "atilde"  ,  227 }, { "auml"    ,  228 }, { "bdquo"   , 8222 },
       { "beta"    ,  946 }, { "brvbar"  ,  166 }, { "bull"    , 8226 },
       { "cap"     , 8745 }, { "ccedil"  ,  231 }, { "cedil"   ,  184 },
       { "cent"    ,  162 }, { "chi"     ,  967 }, { "circ"    ,  710 },
       { "clubs"   , 9827 }, { "cong"    , 8773 }, { "copy"    ,  169 },
       { "crarr"   , 8629 }, { "cup"     , 8746 }, { "curren"  ,  164 },
       { "dArr"    , 8659 }, { "dagger"  , 8224 }, { "darr"    , 8595 },
       { "deg"     ,  176 }, { "delta"   ,  948 }, { "diams"   , 9830 },
       { "divide"  ,  247 }, { "eacute"  ,  233 }, { "ecirc"   ,  234 },
       { "egrave"  ,  232 }, { "empty"   , 8709 }, { "emsp"    , 8195 },
       { "ensp"    , 8194 }, { "epsilon" ,  949 }, { "equiv"   , 8801 },
       { "eta"     ,  951 }, { "eth"     ,  240 }, { "euml"    ,  235 },
       { "euro"    , 8364 }, { "exist"   , 8707 }, { "fnof"    ,  402 },
       { "forall"  , 8704 }, { "frac12"  ,  189 }, { "frac14"  ,  188 },
       { "frac34"  ,  190 }, { "frasl"   , 8260 }, { "gamma"   ,  947 },
       { "ge"      , 8805 }, { "gt"      ,   62 }, { "hArr"    , 8660 },
       { "harr"    , 8596 }, { "hearts"  , 9829 }, { "hellip"  , 8230 },
       { "iacute"  ,  237 }, { "icirc"   ,  238 }, { "iexcl"   ,  161 },
       { "igrave"  ,  236 }, { "image"   , 8465 }, { "infin"   , 8734 },
       { "int"     , 8747 }, { "iota"    ,  953 }, { "iquest"  ,  191 },
       { "isin"    , 8712 }, { "iuml"    ,  239 }, { "kappa"   ,  954 },
       { "lArr"    , 8656 }, { "lambda"  ,  955 }, { "lang"    , 9001 },
       { "laquo"   ,  171 }, { "larr"    , 8592 }, { "lceil"   , 8968 },
       { "ldquo"   , 8220 }, { "le"      , 8804 }, { "lfloor"  , 8970 },
       { "lowast"  , 8727 }, { "loz"     , 9674 }, { "lrm"     , 8206 },
       { "lsaquo"  , 8249 }, { "lsquo"   , 8216 }, { "lt"      ,   60 },
       { "macr"    ,  175 }, { "mdash"   , 8212 }, { "micro"   ,  181 },
       { "middot"  ,  183 }, { "minus"   , 8722 }, { "mu"      ,  956 },
       { "nabla"   , 8711 }, { "nbsp"    ,  160 }, { "ndash"   , 8211 },
       { "ne"      , 8800 }, { "ni"      , 8715 }, { "not"     ,  172 },
       { "notin"   , 8713 }, { "nsub"    , 8836 }, { "ntilde"  ,  241 },
       { "nu"      ,  957 }, { "oacute"  ,  243 }, { "ocirc"   ,  244 },
       { "oelig"   ,  339 }, { "ograve"  ,  242 }, { "oline"   , 8254 },
       { "omega"   ,  969 }, { "omicron" ,  959 }, { "oplus"   , 8853 },
       { "or"      , 8744 }, { "ordf"    ,  170 }, { "ordm"    ,  186 },
       { "oslash"  ,  248 }, { "otilde"  ,  245 }, { "otimes"  , 8855 },
       { "ouml"    ,  246 }, { "para"    ,  182 }, { "part"    , 8706 },
       { "permil"  , 8240 }, { "perp"    , 8869 }, { "phi"     ,  966 },
       { "pi"      ,  960 }, { "piv"     ,  982 }, { "plusmn"  ,  177 },
       { "pound"   ,  163 }, { "prime"   , 8242 }, { "prod"    , 8719 },
       { "prop"    , 8733 }, { "psi"     ,  968 }, { "quot"    ,   34 },
       { "rArr"    , 8658 }, { "radic"   , 8730 }, { "rang"    , 9002 },
       { "raquo"   ,  187 }, { "rarr"    , 8594 }, { "rceil"   , 8969 },
       { "rdquo"   , 8221 }, { "real"    , 8476 }, { "reg"     ,  174 },
       { "rfloor"  , 8971 }, { "rho"     ,  961 }, { "rlm"     , 8207 },
       { "rsaquo"  , 8250 }, { "rsquo"   , 8217 }, { "sbquo"   , 8218 },
       { "scaron"  ,  353 }, { "sdot"    , 8901 }, { "sect"    ,  167 },
       { "shy"     ,  173 }, { "sigma"   ,  963 }, { "sigmaf"  ,  962 },
       { "sim"     , 8764 }, { "spades"  , 9824 }, { "sub"     , 8834 },
       { "sube"    , 8838 }, { "sum"     , 8721 }, { "sup"     , 8835 },
       { "sup1"    ,  185 }, { "sup2"    ,  178 }, { "sup3"    ,  179 },
       { "supe"    , 8839 }, { "szlig"   ,  223 }, { "tau"     ,  964 },
       { "there4"  , 8756 }, { "theta"   ,  952 }, { "thetasym",  977 },
       { "thinsp"  , 8201 }, { "thorn"   ,  254 }, { "tilde"   ,  732 },
       { "times"   ,  215 }, { "trade"   , 8482 }, { "uArr"    , 8657 },
       { "uacute"  ,  250 }, { "uarr"    , 8593 }, { "ucirc"   ,  251 },
       { "ugrave"  ,  249 }, { "uml"     ,  168 }, { "upsih"   ,  978 },
       { "upsilon" ,  965 }, { "uuml"    ,  252 }, { "weierp"  , 8472 },
       { "xi"      ,  958 }, { "yacute"  ,  253 }, { "yen"     ,  165 },
       { "yuml"    ,  255 }, { "zeta"    ,  950 }, { "zwj"     , 8205 },
       { "zwnj"    , 8204 }
     } ;

   /*----------------------------------------------------------------------*/
   for( auto cit = s_in.begin() ; cit != s_in.end() ; ++cit )
   {
      /*--( Not a html symbol )--------------------------------------------*/
      if( *cit != '&' || cit + 1 == s_in.end() || cit + 2 == s_in.end() )
      {  s_out += *cit ; }
      else /*--( A numeric sequence : "&#[x|X]{.};" )----------------------*/
      if(  *( cit + 1 ) == '#' )
      {  /*----------------------------------------------------------------*/
         cit += 2 ;
         /*--( Base used to specify the character code )-------------------*/
         int i_base ;
         if( *cit == 'x' || *cit == 'X' )
         {  i_base = 16 ; ++cit ; }
         else
         {  i_base = 10 ; }
         /*----------------------------------------------------------------*/
         wxString s_num ;
         while( cit != s_in.end() && *cit != ';' ) { s_num += *cit++ ; }
         /*----------------------------------------------------------------*/
         long l_char ;
         s_num.ToLong( &l_char, i_base ) ;
         s_out += wxUniChar( l_char ) ;
         /*----------------------------------------------------------------*/
      }
      else
      {  /*--( Extract next html symbol : between '&' and ';' )------------*/
         wxString s_seq ;
         auto cit_seq = cit + 1 ;
         while( cit_seq != s_in.end() && wxIsalpha( *cit_seq ) )
         {  s_seq += *cit_seq++ ; }
         /*--( Normally a sequence ends with a ';' => non alpha )----------*/
         if( cit_seq == s_in.end() || *cit_seq != ';' )
         {  s_out += *cit ; }
         else
         {  /*--( Search for known html sequence )-------------------------*/
            auto it_map = st_map_char_seq.find( s_seq ) ;
            /*--( Replace if found )---------------------------------------*/
            if( it_map == st_map_char_seq.end() )
            {  s_out.append( cit, cit_seq + 1 ) ; }
            else
            {  s_out += it_map->second ; }
            /*-------------------------------------------------------------*/
            cit = cit_seq ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return s_out ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::url_encode( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   const wxScopedCharBuffer scb_s( s.utf8_str() ) ;
   wxString s_res ;
   /*--( nullptr could be given to "curl_easy_escape" ... )----------------*/
   CURL *curl = curl_easy_init() ;
   if( curl == nullptr ) { return s_res ; }
   /*--( escaping / encoding )---------------------------------------------*/
   char *p_c_res = curl_easy_escape( curl, scb_s.data(), scb_s.length() ) ;
   if( p_c_res != nullptr )
   {  /*-------------------------------------------------------------------*/
      s_res = p_c_res ;
      curl_free( p_c_res ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   curl_easy_cleanup( curl ) ;
   /*----------------------------------------------------------------------*/
   return s_res ;
   /*----------------------------------------------------------------------*/
}

/*--( Check if a string is ok to be an "ini" entry name )------------------*/
int mtf::check_ini_entry_name( const wxString &s_name, wxString &s_err_msg )
{
   /*--( Path separators are seen as such by wx )--------------------------*/
   const char *co_p_c_forbidden = R"=(/\)=" ;
   if( s_name.find_first_of( co_p_c_forbidden ) != wxString::npos )
   {  /*-------------------------------------------------------------------*/
      s_err_msg.Printf( _( "Characters \"%s\" are not allowed" ),
                        co_p_c_forbidden
                      ) ;
      return -1 ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return 0 ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::draw_wrapped( wxDC &dc, const wxString &s )
{
   /*----------------------------------------------------------------------*/
   const int co_i_margin = 8 ;
   wxPoint pt_write( co_i_margin, co_i_margin ) ;
   const int co_i_max_width = dc.GetSize().GetWidth() - 2 * co_i_margin ;
   /*----------------------------------------------------------------------*/
   wxSize size_token ;
   wxString s_current ;
   sc::wxString_cit cit_end = s.begin() ;
   bool boo_fit ;
   size_t sz_len_ok ;
   const int co_i_max_smaller = 3 ;
   int i_smaller = 0 ;
   /*----------------------------------------------------------------------*/
   while( cit_end != s.end() && i_smaller < co_i_max_smaller )
   {
      /*-------------------------------------------------------------------*/
        const int co_i_max_height
      = dc.GetSize().GetHeight() - pt_write.y - co_i_margin ;
      /*-------------------------------------------------------------------*/
      do
      {  /*----------------------------------------------------------------*/
         sz_len_ok = s_current.size() ;
         /*--( Add another word )------------------------------------------*/
         for( ; cit_end != s.end() && *cit_end != ' ' && *cit_end != '-' ;
              ++cit_end
            )
         {  s_current += *cit_end ; }
         /*----------------------------------------------------------------*/
         size_token = dc.GetTextExtent( s_current ) ;
         boo_fit = (    size_token.GetWidth() <= co_i_max_width
                     && size_token.GetHeight() <= co_i_max_height
                   ) ;
         /*--( As it doesn't fit, last word is removed )-------------------*/
         if( !boo_fit && sz_len_ok > 0 )
         {  cit_end -= s_current.size() - sz_len_ok ;
            s_current.Remove( sz_len_ok ) ;
         }
         /*--( Add seperator after first word )----------------------------*/
         if( boo_fit && cit_end != s.end() )
         {  s_current += *cit_end++ ; }
         /*----------------------------------------------------------------*/
      } while( boo_fit && cit_end != s.end() ) ;

      /*-------------------------------------------------------------------*/
      if( !boo_fit && sz_len_ok == 0 && i_smaller < co_i_max_smaller )
      {
         /*-----------------------------------------------------------------+
         ! Test smaller font if only one word doesn't fit                   !
         ! The font used will remain "smaller" because it seems visually    !
         ! nicer to look at                                                 !
         +-----------------------------------------------------------------*/
         bool boo_ok = false ;
         /*----------------------------------------------------------------*/
         for( ; !boo_ok && i_smaller < co_i_max_smaller ; ++i_smaller )
         {  /*-------------------------------------------------------------*/
            dc.SetFont( dc.GetFont().Smaller() ) ;
            /*-------------------------------------------------------------*/
            size_token = dc.GetTextExtent( s_current ) ;
            boo_ok = (    size_token.GetWidth()  <= co_i_max_width
                       && size_token.GetHeight() <= co_i_max_height
                     ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }

      /*-------------------------------------------------------------------*/
      dc.DrawText( s_current, pt_write ) ;
      pt_write.y += size_token.GetHeight() ;
      /*-------------------------------------------------------------------*/
      s_current.clear() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
#ifdef __WXMSW__
/*-------------------------------------------------------------------------*/
#ifndef MTF_MAKEFOURCC
#define MTF_MAKEFOURCC(ch0,ch1,ch2,ch3) \
  ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) | \
   ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24))
#endif /* MTF_MAKEFOURCC */
/*-------------------------------------------------------------------------*/
int mtf::msw_exe_bitness( const wxString &s_filename )
{
   /*----------------------------------------------------------------------*/
   wxLogNull no_log ; // Suppress useless warnings
   wxFile   fi( s_filename ) ;
   wxUint32 dw_offset ;
   wxUint32 dw_sign ;
   IMAGE_FILE_HEADER ifh ;

   /*--( Access the right data )-------------------------------------------*/
   if(    !fi.IsOpened()
       || fi.Seek( 0x3c ) == wxInvalidOffset
       || fi.Read( &dw_offset, sizeof( dw_offset ) ) == wxInvalidOffset
       || fi.Seek( dw_offset ) == wxInvalidOffset
       || fi.Read( &dw_sign, sizeof( dw_sign ) ) == wxInvalidOffset
       || dw_sign != MTF_MAKEFOURCC( 'P','E','\0','\0' )
       || fi.Read( &ifh, sizeof( ifh ) ) ==  wxInvalidOffset
     )
   {  return -1 ; }

   /*----------------------------------------------------------------------*/
   switch( ifh.Machine )
   {  /*-------------------------------------------------------------------*/
      case IMAGE_FILE_MACHINE_I386  : return 32 ;
      /*-------------------------------------------------------------------*/
      case IMAGE_FILE_MACHINE_IA64  :
      case IMAGE_FILE_MACHINE_AMD64 : return 64 ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return -2 ;
   /*----------------------------------------------------------------------*/
}
#endif

/*-------------------------------------------------------------------------*/
wxString mtf::read_m3u_line( wxTextInputStream &tis, int &i_line_num )
{
   /*----------------------------------------------------------------------*/
   wxString s ;
   /*--( Empty and comment lines are skipped )-----------------------------*/
   while(    !tis.GetInputStream().Eof()
          && (    ( s = tis.ReadLine() ).Trim().empty()
               || s.StartsWith( "#EXTREM:" )
             )
        )
   {  ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   if( !tis.GetInputStream().Eof() ) { ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   return s ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::read_line( wxTextInputStream &tis, int &i_line_num )
{
   /*----------------------------------------------------------------------*/
   wxString s ;
   /*--( Empty and comment lines are skipped )-----------------------------*/
   while(    !tis.GetInputStream().Eof()
          && (    ( s = tis.ReadLine() ).Trim().empty()
               || s.StartsWith( "#" )
             )
        )
   {  ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   if( !tis.GetInputStream().Eof() ) { ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   return s ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static size_t st_libcurl_header_callback( char *buffer, size_t size,
                                          size_t nitems, void *userdata
                                        )
{
   /*----------------------------------------------------------------------*/
   wxString *p_s_url = static_cast< wxString * >( userdata ) ;
   const size_t co_sz_size = nitems * size ;
   const wxString co_s_data( buffer, co_sz_size ) ;
   /*----------------------------------------------------------------------*/
   sc::match( co_s_data, R"=(\mlocation: (.*)\M)=", p_s_url ) ;
   /*----------------------------------------------------------------------*/
   return co_sz_size ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::unshort_url( wxString &s_url, wxString *p_s_err_msg )
{
   /*----------------------------------------------------------------------*/
   CURL *curl = curl_easy_init() ;
   int i_ret = 0 ;
   /*----------------------------------------------------------------------*/
   if( curl == nullptr ) { return -1 ; }
   /*----------------------------------------------------------------------*/
   wxCharBuffer cb_url = s_url.ToAscii() ;
   curl_easy_setopt( curl, CURLOPT_URL, cb_url.data() ) ;
   sc::CDownload::st_init_proxy( curl ) ;
   curl_easy_setopt( curl, CURLOPT_HEADER, 1 ) ;
   curl_easy_setopt( curl, CURLOPT_HEADERDATA, &s_url ) ;
   curl_easy_setopt( curl, CURLOPT_HEADERFUNCTION,
                     st_libcurl_header_callback
                   ) ;
   curl_easy_setopt( curl, CURLOPT_FOLLOWLOCATION, 0 ) ;

   /*-----------------------------------------------------------------------+
   ! GO ! GO ! GO !                                                         !
   ! Note that the "write error" error can happen for "periscope" ...       !
   +-----------------------------------------------------------------------*/
   sc::CDownloadBase::e_err_code err_code
     = sc::CDownloadBase::st_curle_to_de( curl, curl_easy_perform( curl ) ) ;
   if(    err_code != sc::CDownloadBase::DE_NO_ERROR_DONE
       && err_code != sc::CDownloadBase::DE_WRITE_ERROR
     )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = sc::CDownloadBase::st_get_err_msg( err_code ) ; }
      /*-------------------------------------------------------------------*/
      i_ret = -2 ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   curl_easy_cleanup( curl ) ;
   /*----------------------------------------------------------------------*/
   return i_ret ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::wxs2int( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   long l_val = 0 ;
   /*----------------------------------------------------------------------*/
   s.ToLong( &l_val ) ;
   return l_val ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::rand( int i_min, int i_max )
{
   /*----------------------------------------------------------------------*/
   if( i_min >= i_max ) { return i_min ; }
   /*----------------------------------------------------------------------*/
   return i_min + std::rand() % ( i_max - i_min + 1 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
size_t CConv_UTF8_8859_1::ToWChar( wchar_t *dst, size_t dstLen,
                                   const char *src, size_t srcLen
                                 ) const
{
   /*----------------------------------------------------------------------*/
   size_t sz_ret_conv = wxConvUTF8.ToWChar( dst, dstLen, src, srcLen ) ;
   /*-----------------------------------------------------------------------+
   ! If the last conversion failed and the available "temporary" space is   !
   ! filled (seems to be 4 bytes) the 8859_1 conversion is done.            !
   +-----------------------------------------------------------------------*/
   if( sz_ret_conv == wxCONV_FAILED && srcLen >= dstLen - 1 )
   {  return wxConvISO8859_1.ToWChar( dst, dstLen, src, srcLen ) ; }
   /*----------------------------------------------------------------------*/
   return sz_ret_conv ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file matelefree.cpp                         !
+==========================================================================*/
